// =================================== GLOOBUS =============================================== //
// ==																						== //	
// ==  Gloobus, a quicklook for linux														== //
// ==																						== //
// ==	Project site: http://launchpad.net/gloobus 											== //
// ==																						== //
// ==	This is a quicklook for linux, it is based on a plugin architecture so everyone can == //
// ==   develop his own plugin to preview any filetype and do whatever it wants				== //
// ==																						== //
// ==   Plase, consider to donate cause it takes me a lot of time and efforts, and you know,== //
// ==   getting a recompense for you job allways makes you do it better						== //
// ==																						== //
// ==																						== //
// ==	Auhtor: Jordi Hernandez Puigdellívol <guitarboy000@gmail.com>						== //
// =========================================================================================== //   

#ifndef _IGLOOBUS_
#define _IGLOOBUS_

#include <map>
#include <string>
#include <vector>

#include <GL/gl.h>
#include <GL/glu.h>

using namespace std;

enum gloobus_type { GLOOBUS, PREVIEW};

class iGloobus 
{

	protected:
		GLuint 				m_textureID;			// Variable of the abstract class
		string 				m_filename;
		gloobus_type 		m_gloobus_type;

	public:
		iGloobus() : m_textureID(0) {}				// Initialize m_textureID
		
		// === FUNCTIONS THAT EACH PLUGIN MUST IMPLEMENT === //	
		virtual void				load() = 0;					 // Load the file		//TODO: INIT_GLOOBUS or INIT_PREVIEW
		virtual void				init() = 0;					 // Initialize the plugin
		virtual void				end() = 0;					 // Destroy the plugin
		virtual void				render() = 0;				 // Render function	
		virtual void				update(char key[]) = 0;		 // Update function
		virtual void				get_size(int *w, int *h) = 0;// To know the size of the GL window (on preview)
		 

		// ====== FUNCTIONS FROM THE ABSTRACT CLASS ========== //
		void set_filename(string filename) 				{    m_filename = filename;		}
		
		string get_filename()							{   return m_filename;  		}
		
		void set_gloobus_type(gloobus_type type)		{    m_gloobus_type = type;		}
		
		//TODO: ADD The max and min sizes here!	   
};

//==============  CLASS FACTORIES DEFINITIONS================================= //
struct plugin_filetype
{
		string name;			//Plugin Name
		string filetype;		//Filetype that plugins reads
		int priority;			//Priority of use this plugin before another
};

typedef iGloobus* create_t();								//CREATE PLUGIN
typedef void destroy_t(iGloobus*);							//DESTROY PLUGIN
typedef int getVersion_t();									//GET PLUGIN_MANAGER VERSION
	
extern map<string, create_t *, less<string> > factory;		   //MAP WITH NAME AND CONSTRUCTOR
extern map<string, destroy_t *, less<string> > factoryDestroy; //MAP WITH NAME AND DESTRUCTOR
extern vector<plugin_filetype> pluginVector;				   //VECTOR TO STORE WICH FILETYPE READ EACH PLUGIN

#endif

