#include "iMp3.h"

#define MAX_W 900
#define MAX_H 700

#define MIN_W 300		// MIN SIZE, if the image is smallest than it
#define MIN_H 300		// It goes fucking slow!!


// ============================= INIT ========================= //
void iMp3::init()
{
	printf("Initializing iPhoto (Load Texture)\n");
	load_texture();
	
	if(m_gloobus_type == PREVIEW)		init_music();		// Load Music if we are in preview

}
// ============================= END ========================== //
void iMp3::end()
{
	printf("Ending iPhoto\n");
	glDeleteTextures(1,&m_textureID);
	SDL_FreeSurface(m_photo);
	
	if(m_gloobus_type == PREVIEW) free_music();
}

// ============================= UPDATE ========================== //


// ============================= RENDER ========================== //
void iMp3::render()
{
		float ratio;
		ratio = (float)m_photo->w / (float)m_photo->h;
		
		if(m_coverType == GL_RGBA) {glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);}
		//TEXTURE
		glEnable(GL_TEXTURE_RECTANGLE_ARB);
		glBindTexture( GL_TEXTURE_RECTANGLE_ARB, m_textureID );
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		//QUAD
		glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 0.0f); 				glVertex2f( -1*ratio, 1);	
				glTexCoord2f(m_photo->w, 0.0f); 		glVertex2f( 1*ratio, 1);
				glTexCoord2f(m_photo->w, m_photo->h); 	glVertex2f( 1*ratio, -1);
				glTexCoord2f(0.0f, m_photo->h);			glVertex2f( -1*ratio, -1);
		glEnd();
		glDisable(GL_TEXTURE_RECTANGLE_ARB);

}
// ============================= GET SIZE ======================== //
void iMp3::get_size(int *w, int *h)
{
		
		*w = m_photo->w;	// In the case doesn't mach any if
		*h = m_photo->h;
		
		// MAX SIZE
		if(*w > MAX_W)
		{
			float value = MAX_W/float(*w);
			*w = *w * value;
			*h = *h * value;
		}
		if(*h > MAX_H)
		{
			float value = MAX_H/float(*h);					
			*w = *w * value;
			*h = *h * value;			
		}
		// MIN SIZE
		// IF THE SIZE IS SMALLEST THAN 300x300 it runs very slow!!
		if(*w < MIN_W)
		{
			float value = MIN_W/float(*w);						
			*w = *w * value;
			*h = *h * value;			
		}

		if(*h < MIN_H)
		{
			float value = MIN_H/float(*h);				
			*w = *w * value;
			*h = *h * value;			
		}
}

// ============================= LOAD MP3 ============================ //
void iMp3::load()
{	
	m_photo = NULL;
	m_coverType = GL_RGB;
	
	printf("Loading Mp3: %s\n",m_filename.c_str());
	
	TagLib::MPEG::File file(m_filename.c_str());
	TagLib::ID3v2::Tag *tagv2 = file.ID3v2Tag();
	if(tagv2)
	{
	
		TagLib::ID3v2::FrameListMap tags  = tagv2->frameListMap();
		TagLib::ID3v2::FrameList pictures = tags["APIC"];
		printf("4\n");
		
		if(!pictures.isEmpty())				 // Thumbnail exists
		{	
			
		   	printf("Loading attached Cover\n");				// Just get the front() picture
		    TagLib::ID3v2::AttachedPictureFrame *picture = 
			static_cast<TagLib::ID3v2::AttachedPictureFrame*>(pictures.front());
		    
			TagLib::ByteVector pictureData  = picture->picture();
		   
		    int size   = pictureData.size();
		    char *data = pictureData.data();
		
			SDL_RWops *rw;
			rw = SDL_RWFromMem(data, size);
			m_photo = IMG_LoadJPG_RW(rw);
			
		}
	}
	if(m_photo == NULL)		//If we can't load the image from mp3, search for cover.jpg on folder
	{
		string text = m_filename;
		text = m_filename.substr(0,m_filename.rfind('/',m_filename.length()));	// Cut just filename
		text = text +"/cover.jpg";
		printf("Tring to load... %s\n",text.c_str());
		m_photo = IMG_Load(text.c_str());
	}

	if(m_photo == NULL)		//If we can't load the image from mp3, load default cover
	{
		printf("Loading noCover: %s\n","nocover.png");
		m_photo = IMG_Load("/usr/share/gloobus/nocover.png");
		m_coverType = GL_RGBA;		
	}
}
// ============================= LOAD TEXTURE ======================== //
void iMp3::load_texture()
{
	// DO This after GL_SETUP -> In main
	GLuint texture;
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_RECTANGLE_ARB, texture);
	glTexImage2D( GL_TEXTURE_RECTANGLE_ARB, 0, m_coverType, m_photo->w, m_photo->h, 0,
				       m_coverType, GL_UNSIGNED_BYTE, m_photo->pixels );
				
	m_textureID = texture;
}


// ========================= INIT MUSIC ======================================== //
void iMp3::init_music()
{	
	printf("Loading Music: %s\n",m_filename.c_str());
	m_music.music = NULL;
	m_music.audio_rate = 22050;
	m_music.audio_format = AUDIO_S16; 
	m_music.audio_channels = 2;
	m_music.audio_buffers = 4096;
	
	if(Mix_OpenAudio(m_music.audio_rate, m_music.audio_format, m_music.audio_channels, m_music.audio_buffers)) 
	{
		printf("Unable to open audio!\n");
		//exit(1);
	}
	else
	{
	m_music.music = Mix_LoadMUS(m_filename.c_str());
	Mix_PlayMusic(m_music.music, 0);
	}

}
//============================= PREVIEW : FREE MUSIC =======================================//
void iMp3::free_music()
{
	Mix_HaltMusic();
	Mix_FreeMusic(m_music.music);
	m_music.music = NULL;
	Mix_CloseAudio();   //TEMP
}
