#include <stdlib.h>
#include <string.h>
#include "../timer.h"
#include "video.h"


GLuint program;

static char* text =
"!!ARBfp1.0\n"

"PARAM vr = { 1.0000,  0.0000,  1.4022 };\n"
"PARAM vg = { 1.0000, -0.3457, -0.7145 };\n"
"PARAM vb = { 1.0000,  1.7710,  0.0000 };\n"

"TEMP yuv;\n"

"TEX yuv, fragment.texcoord[0], texture, 2D;\n"

"DP3 result.color.r, yuv, vr;\n"
"DP3 result.color.g, yuv, vg;\n"
"DP3 result.color.b, yuv, vb;\n"
"MOV result.color.a, yuv.a;\n"

"END\n";



static GLfloat Matrix_YCbCr2RGB[16] = {
     1, 1, 1, 0,
     0, -0.344136, 1.773, 0,
     1.402, -0.714136, 0, 0,
     0, 0, 0, 0
     };

Video* create_video(char* filename) {
    Video* video = malloc(sizeof(Video));
    video->position.posx = 1;
    video->position.posy = 0;
    video->position.posz = 0;
    video->quad = create_quad();
    video->filename = filename;
    video->buffer = NULL;

    av_register_all();
    if(av_open_input_file(&video->format_ctx, video->filename, NULL, 0,
        NULL) != 0) {
        destroy_video(video);
        exit(1);    
    }

    if(av_find_stream_info(video->format_ctx) < 0) {
        destroy_video(video);
        exit(1);
    }
    
    video->video_stream = -1;
    int i;
    for(i = 0; i < video->format_ctx->nb_streams; i++) {
        if(video->format_ctx->streams[i]->codec->codec_type == 
            CODEC_TYPE_VIDEO) {
            video->video_stream = i;
            //printf("Found video stream at %d\n", i);
            break;
        }
    }
    video->codec_ctx = 
        video->format_ctx->streams[video->video_stream]->codec;

    video->codec = avcodec_find_decoder(video->codec_ctx->codec_id);
    if(video->codec == NULL) {
        //fprintf(stderr, "Could not find codec for video %s", filename);
        exit(1);
    }

    if(video->codec == NULL)
        return NULL;

    if(avcodec_open(video->codec_ctx, video->codec)<0) {
        //fprintf(stderr, "Could not open codec for video %s", filename);
        exit(1);
    }
    video->frame = avcodec_alloc_frame();
    video->frame_rgb = avcodec_alloc_frame();
    glGenTextures(1, &video->texture);
    
    if(video->codec_ctx->pix_fmt == PIX_FMT_YUV420P) {
        //printf("Width: %d Height: %d\n", video->codec_ctx->width,
        //    video->codec_ctx->height);
        //printf("PIX_FMT_YUV420P\n");
        //glPixelStorei(GL_UNPACK_ROW_LENGTH, video->codec_ctx->width);
        //glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    }
    
    video->img_convert_ctx = sws_getContext(
        video->codec_ctx->width,
        video->codec_ctx->height,
        video->codec_ctx->pix_fmt,
        video->codec_ctx->width, 
        video->codec_ctx->height,
        PIX_FMT_RGB24, SWS_BICUBIC,
        NULL,
        NULL, 
        NULL);

    int num_bytes;
    num_bytes = avpicture_get_size(PIX_FMT_RGB24, video->codec_ctx->width,
        video->codec_ctx->height);
    video->buffer = (uint8_t *)av_malloc(num_bytes * sizeof(uint8_t));

    avpicture_fill((AVPicture *)video->frame_rgb, video->buffer, 
        PIX_FMT_RGB24, video->codec_ctx->width, video->codec_ctx->height);
 

    /*
    glGenProgramsARB(1, &program);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, program);

    glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB,
                       GL_PROGRAM_FORMAT_ASCII_ARB, strlen(text), text);

    if (glGetError() == GL_INVALID_OPERATION)
        fprintf(stderr, "%s\n", glGetString(GL_PROGRAM_ERROR_STRING_ARB));
    */

    return video;
}

void destroy_video(Video* video) {
    /*
    if(video == NULL)
        return;
    */
    //destroy_quad(video->quad);
    glDeleteTextures(1, &video->texture);
    
    /*
    if(video->buffer != NULL)
        av_free(video->buffer);
    if(&video->packet != NULL)
        av_free(&video->packet);
    if(video->frame != NULL)
        av_free(video->frame);
    if(video->frame_rgb != NULL)
        av_free(video->frame_rgb);
    if(video->codec_ctx != NULL)
        avcodec_close(video->codec_ctx);
    if(video->format_ctx != NULL)
        av_close_input_file(video->format_ctx);
    
    */
    free(video);
}

int update_video(Video* video) {

    //clock_t render_frame_time = start_timer();
    if(av_read_frame(video->format_ctx, &video->packet) < 0) {
        //fprintf(stderr, "Error reading frame\n");
        return -1;
    }
    int frame_finished = 0;
    if(video->packet.stream_index == video->video_stream) {

        avcodec_decode_video(video->codec_ctx, video->frame, 
            &frame_finished, video->packet.data, video->packet.size);

        if(!frame_finished) 
            return 0;

        //glPixelStorei(GL_UNPACK_ROW_LENGTH, video->frame->linesize[0]);
        //glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        glBindTexture(GL_TEXTURE_2D, video->texture);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
        
        //clock_t t1 = start_timer();

        sws_scale(video->img_convert_ctx, video->frame->data, 
            video->frame->linesize, 0, video->codec_ctx->height, 
            video->frame_rgb->data, video->frame_rgb->linesize);
        
        glTexImage2D(GL_TEXTURE_2D, 0, 3,
            video->codec_ctx->width, video->codec_ctx->height, 0,
            GL_RGB, GL_UNSIGNED_BYTE, 
            video->frame_rgb->data[0]);
        
        //print_elapsed_time(t1, "Texture time:");
        
       /*
        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0,
            video->codec_ctx->width, 
            video->codec_ctx->height, 
            GL_RGB, GL_UNSIGNED_BYTE, video->frame->data[0]);
        */
       return 1;
    }
    return 0;
}

void draw_video(Video* video) {

    //clock_t t = start_timer();
    glEnable(GL_TEXTURE_2D);

    //draw_position(&video->position);

    /*
    glEnable(GL_FRAGMENT_PROGRAM_ARB);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, program);
    */
    /*
    int count = 0;
    while(render_frame(video) == 0) {
        count++;
    }
    */
    //printf("Found video frame at: %d\n", count);
    
    /*
    glMatrixMode(GL_COLOR);
    glLoadMatrixf(Matrix_YCbCr2RGB);
    glPixelTransferf(GL_GREEN_BIAS, -0.5);
    glPixelTransferf(GL_BLUE_BIAS, -0.5);
    */  
    
    glBindTexture(GL_TEXTURE_2D, video->texture);
    draw_quad(video->quad);
    glDisable(GL_TEXTURE_2D);
    //print_elapsed_time(t, "draw_time:");
}
