// =================================== GLOOBUS =============================================== //
// ==																						== //	
// ==  Gloobus, a quicklook for linux														== //
// ==																						== //
// ==	Project site: http://launchpad.net/gloobus 											== //
// ==																						== //
// ==	This is a quicklook for linux, it is based on a plugin architecture so everyone can == //
// ==   develop his own plugin to preview any filetype and do whatever it wants				== //
// ==																						== //
// ==   Plase, consider to donate cause it takes me a lot of time and efforts, and you know,== //
// ==   getting a recompense for you job allways makes you do it better						== //
// ==																						== //
// ==																						== //
// ==	Auhtor: Jordi Hernandez Puigdellívol <guitarboy000@gmail.com>						== //
// =========================================================================================== //   

#include <iostream>
#include <dlfcn.h>
#include <SDL/SDL_syswm.h>

#include "preview.h"
#include "pluginManager.h"

#define KEY_DELAY   500
#define KEY_REPEAT  80

using namespace std;

map<string, create_t *, less<string> >  factory;		//Global variable that plugin uses to register
map<string, destroy_t *, less<string> > factoryDestroy;	//Global variable that plugin uses to destroy
vector<plugin_filetype> pluginVector;					//Global variable that plugin uses to say what filetype it loads

// ============================ SETUP OPENGL================================================= //
void GL_Setup(int width, int height)
{
		glViewport( 0, 0, width, height );
		glMatrixMode( GL_PROJECTION );
		glEnable( GL_DEPTH_TEST );
		glEnable(GL_BLEND);
		gluPerspective( 45, (float)width/height, 0.1, 100 );
		glMatrixMode( GL_MODELVIEW );	
}

//=============================================================================================//
int main(int argc, char* argv[]) {
      
    string filename;

	if(argc!=2)									// Obtain filename to preview
	{
		printf("Preview needs a file\n");
		exit(-1);
	}
	else { 	filename = argv[1];	}
	
	// ======================== LOAD PLUGINS ========================= //
    pluginManager PManager;
	PManager.load_plugins();					// Load all plugins on /usr/share/gloobus/plugins
	
	int w,h;
	string key = PManager.search_plugin_for_file(filename);	//Search the plugin to use for this file
	if(key == "NULL"){ 
		printf("This filetype has no preview function\n");
		exit(-1);
	}
	
	// ======================== SELECT PLUGIN ===================== //
	
	printf("Using Plugin:%s\n",key.c_str());
	iGloobus * gloobus = factory[key]();			// Constructor

	gloobus->set_filename(filename);
	gloobus->set_gloobus_type(PREVIEW);				// Set preview 
	gloobus->load();								// Load the file
	gloobus->get_size(&w,&h);					
	printf("SIZE: %d, %d\n",w,h);
	
	// ======================= SET VIDEO ========================== //	
	
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);					 // Initialize SDL with best video mode
	const SDL_VideoInfo* info = SDL_GetVideoInfo();	
	int vidFlags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWPALETTE | SDL_RESIZABLE| SDL_NOFRAME;
	
	if (info->hw_available) {vidFlags |= SDL_HWSURFACE;}
	else 					{vidFlags |= SDL_SWSURFACE;}

	int bpp = info->vfmt->BitsPerPixel;

	SDL_Surface *screen = SDL_SetVideoMode(w, h, bpp, vidFlags);
	gtv_center_window(screen);
	GL_Setup(w, h);
	SDL_EnableKeyRepeat(KEY_DELAY, KEY_REPEAT);	// Enable Key Repeat

	// ========================================================== //
	// ===== Main bucle   INIT - [ UPDATE - RENDER ] - QUIT	===== //

	Preview* previewManager = new Preview();	

	gloobus->init();				// Initialize	
	previewManager->set_size(w,h);	// Set size
	previewManager->init();			// Initialize
	

	while(previewManager->update(gloobus))	
	{
		//TODO: Optimize this bucle to not load 100% cpu
		//if(engine->draw())		// ONLY RENDER IF THERE ARE UPDATES
			previewManager->render(gloobus);
			sleep(0.9);			// TO NOT USE 100% CPU
	}
	// ========================================================== //	

	delete previewManager;
	
	gloobus->end();									// End Plugin
	factoryDestroy[key](gloobus);					// Destructor
	PManager.unload_plugins();						// Unload Plugins
	
	SDL_Quit();
}


